import { ILogger } from '@lskjs/log';
import { IEventEmitter } from './IEventEmitter.types.mjs';

interface IModuleLifecycle {
    /**
     * Дата создания модуля
     */
    create?: Date;
    /**
     * Дата начала инициализации модуля
     */
    initStart?: Date;
    initFinish?: Date;
    /**
     * Дата начала запуска модуля
     */
    runStart?: Date;
    runFinish?: Date;
    /**
     * Дата начала остановки модуля
     */
    stopStart?: Date;
    stopFinish?: Date;
    /**
     * Автоматический запуск модуля родителем
     */
    autorun?: boolean;
}
/**
 * ворфлоу работы модуля
 */
interface IModuleWithLifecycle {
    name?: string;
    /**
     * Воркфлоу работы модуля
     */
    __lifecycle: IModuleLifecycle;
    /**
     * setProp -- правило заподнение класса пропом
     */
    setProp(key: string, value: IModuleProp): void;
    /**
     * setProps -- заполнение класса пропсами
     */
    setProps(...propsArray: IModuleProps[]): void;
    /**
     * init -- по сути асинхронный конструктор, следует вызывать сразу после new, и дожидаться перед использованием
     */
    __init(): Promise<void>;
    init(): Promise<void>;
    /**
     * run -- запуск основных функций класса
     * если у класса не сделан init, он сделает
     * оперируем с переменными, запросы не делаем
     */
    start(): Promise<void>;
    __run(): Promise<void>;
    run(): Promise<void>;
    /**
     * stop -- grace остановка класса
     * выключаем все подключение и готовимся умирать
     */
    __stop(): Promise<void>;
    stop(): Promise<void>;
}
interface IModuleWithСonfig extends IModuleWithLifecycle {
    config?: {
        [name: string]: any;
    };
}
interface IModuleWithLog extends IModuleWithСonfig {
    log?: ILogger;
    createLog(): ILogger;
}
interface IModuleWithEE extends IModuleWithLog, IEventEmitter {
    /**
     * евент эмиттер
     */
    ee?: IEventEmitter;
    /**
     * создаем эвент эмиттер
     */
    createEe(): IEventEmitter;
}
type IModuleGetter = (m: IModule) => any;
interface IModuleWithSubmodules extends IModuleWithEE {
    /**
     * private
     */
    __availableModules?: IAsyncModuleKeyValue;
    /**
     * private
     */
    __initedModules?: IModuleKeyValue;
    /**
     * private
     */
    __getModules(): Promise<IAsyncModuleKeyValue>;
    parent?: IModule;
    modules?: IAsyncModuleKeyValue;
    getModules(): IAsyncModuleKeyValue | Promise<IAsyncModuleKeyValue>;
    module(nameOrNames: string | string[], options?: {
        run?: boolean;
        throw?: boolean;
        getter?: IModuleGetter;
    }): Promise<null | IModule | IModuleKeyValue>;
}
interface IModuleWithApp extends IModuleWithSubmodules {
    app?: IApp;
}
type IModule = IModuleWithApp;
type IModuleProp = any;
type IModuleProps = {
    [key: string]: IModuleProp;
} | IApp;
interface IModuleConstructor<TT extends IModule> {
    /**
     * создать инстанс и проинициализировать его
     */
    create<T extends IModule>(this: IModuleConstructor<T>, ...propsArray: IModuleProps[]): Promise<T>;
    /**
     * создать инстанс, проинициализировать и запустить
     */
    createAndRun<T extends IModule>(this: IModuleConstructor<T>, ...propsArray: IModuleProps[]): Promise<T>;
    start<T extends IModule>(this: IModuleConstructor<T>, ...propsArray: IModuleProps[]): Promise<T>;
    new (): TT;
}
type IModuleKeyValue = {
    [name: string]: IModule | null;
};
/**
 * AsyncModule
 */
type IAsyncModuleProps = {
    Module: any;
    [key: string]: IModuleProp;
};
type IAsyncModule = IModule | any | IAsyncModuleProps | Promise<IAsyncModule>;
type IAsyncModuleKeyValue = {
    [name: string]: IAsyncModule;
};
/**
 * App
 */
type IAppModel = any;
type IAppModelKeyValue = {
    [name: string]: IAppModel;
};
interface IApp extends IModule {
    models: IAppModelKeyValue;
    model(name: string): Promise<IAppModel>;
    model(names: string[]): Promise<IAppModelKeyValue>;
}

export { IApp, IAppModel, IAppModelKeyValue, IAsyncModule, IAsyncModuleKeyValue, IAsyncModuleProps, IModule, IModuleConstructor, IModuleKeyValue, IModuleLifecycle, IModuleProp, IModuleProps, IModuleWithApp, IModuleWithEE, IModuleWithLifecycle, IModuleWithLog, IModuleWithSubmodules, IModuleWithСonfig };
